-- GlobalOptions.lua : Options config
if select(6, GetAddOnInfo("PitBull4_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local _G = getfenv(0)
local PitBull4 = _G.PitBull4
local PitBull4_VisualHelpers= PitBull4:GetModule("VisualHelpers")
local L = PitBull4.L
local UpdateFrames = PitBull4.Options.UpdateFrames

local MULTISELECT_CONTROL = "Dropdown"

local CURRENT_HELPER
local AURAS_GROUP = {}


-- :::::::::::: LOCALS ::::::::::::: -------------------------------------------------------------------------
local function add_helper(new_helper_name) 
	local helpers = PitBull4_VisualHelpers.db.profile.global.helpers
	helpers[new_helper_name] = {
		name = new_helper_name,
		auras = {},
	}
end

local function auras_count()
	local auras = PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER].auras
	if not auras then return 0 end
	local num = 0
	for name in pairs(auras) do
		num = num + 1
	end
	return num
end

local function helpers_count()
	local helpers = PitBull4_VisualHelpers.db.profile.global.helpers
	if not helpers then return 0 end
	local num = 0
	for name in pairs(helpers) do
		num = num + 1
	end
	return num
end

local function build_aura_list()
	local auras = PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER].auras

	for k, v in pairs(AURAS_GROUP) do
		AURAS_GROUP[k] = nil
	end

	if auras_count() < 1 then
		AURAS_GROUP['__DUMMY__'] = PitBull4_VisualHelpers:CreateNewAuraGlobalOptions(' ')
	else
		for k, v in pairs(auras) do
			AURAS_GROUP[k] = PitBull4_VisualHelpers:CreateNewAuraGlobalOptions(k)
		end
	end 
	LibStub("AceConfigRegistry-3.0"):NotifyChange("PitBull4")
end

-- :::::::::::: PitBull4_VisualHelpers ::::::::::::: -------------------------------------------------------------------------
function PitBull4_VisualHelpers:CreateDefaultGlobalHelpers()
	PitBull4_VisualHelpers.db.profile.global.helpers = {
		['Tank Cooldowns'] = {
			name = 'Tank Cooldowns',
			auras = {
				["Shield Wall"] = {},
				["Survival Instincts"] = {},
				["Frenzied Regeneration"] = {},
				["Divine Protection"] = {},
				["Pain Suppression"] = {},
				["Power Word: Barrier"] = {},
				["Ardent Defender"] = {},
				["Guardian of Ancient Kings"] = {},
				["Enraged Regeneration"] = {},
				["Barkskin"] = {},
				["Anti-Magic Shell"] = {},
				["Icebound Fortitude"] = {},
				["Hand of Sacrifice"] = {},
				["Divine Guardian"] = {},
				["Guardian Spirit"] = {},
				["Safeguard"] = {},
				["Divine Shield"] = {},
			},
		},
		['Bad Stuff'] = {
			name = 'Bad Stuff',
			auras = {
				["Corruption: Accelerated"] = {},
				["Stormbolt"] = {},
				["Low Health"] = {},
				["Waterlogged"] = {},
				["Corruption: Malformation"] = {},
				["Corruption: Absolute"] = {},
				["Cho's Blast"] = {},
				["Gall's Blast"] = {},
				["Elemental Stasis"] = {},
			},
		},
	}
	return PitBull4_VisualHelpers.db.profile.global.helpers
end

function PitBull4_VisualHelpers:GetHelpersNames() 
	local helpers = PitBull4_VisualHelpers.db.profile.global.helpers
	local t = {}
	for n in pairs(helpers) do
		t[n] = n
	end 
	return t
end

PitBull4_VisualHelpers:SetGlobalOptionsFunction(function(self)
	return 'helpers_editor', {
		type = 'group',
		childGroups = 'tab',
		name = L['Helpers editor'],
		desc = L['Configure the visual helpers you will want in your layout.'],
		args = PitBull4_VisualHelpers:GetHelpersEditor(),
	}
end)

function PitBull4_VisualHelpers:CreateNewAuraGlobalOptions(aura_name)
	return {
		type = 'group',
		name = aura_name,
		childGroups = 'tree',
		args = {
			auraname = {
				type = 'input',
				name = L['Aura name'],
				desc = L['Change the name of the current aura here.'],
				get = function(info,key)
					return aura_name
				end,
				set = function(info,value)
					local auras = PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER].auras
					if auras and auras[aura_name] and not auras[value] then
						auras[value] = {
							zone = auras[aura_name].zone,
							priority = auras[aura_name].priority,
						}
						if auras[aura_name] then auras[aura_name] = nil end
						build_aura_list()
						PitBull4_VisualHelpers:UpdateIndicators()
						UpdateFrames()
					end					
				end,
				order = 1,
			},
			aurazone = {
				type = 'input',
				name = L['Zone filter'],
				desc = L['Zone filter for current aura, leave blank for no filter.'],
				get = function(info,key)
					local auras = PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER].auras
					if auras and auras[aura_name] then
						return auras[aura_name].zone or ''
					else
						return ''
					end
				end,
				set = function(info,value)
					local auras = PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER].auras
					if auras and auras[aura_name] then
						auras[aura_name].zone = value
						PitBull4_VisualHelpers:UpdateIndicators()
						UpdateFrames()
					end
				end,
				order = 2,
			},
			aurapriority = {
				type = 'range',
				name = L["Priority"],
				desc = L["Priority of the current aura, higher priorities display over lower ones regardless of duration."],
				softMin = 0,
				softMax = 100,
				step = 1,
				bigStep = 5,
				order = 3,
				get = function(info)
					local auras = PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER].auras
					if auras and auras[aura_name] then
						return auras[aura_name].priority or 0
					else return 0 end
				end,
				set = function(info, value)
					local auras = PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER].auras
					if auras and auras[aura_name] then
						auras[aura_name].priority = value
						PitBull4_VisualHelpers:UpdateIndicators()
						UpdateFrames()
					end
				end,
			},
			auraselfonly = {
				type = 'toggle',
				name = L["Only mine"],
				desc = L["Only show when this buff/debuff has been cast by you"],
				order = 4,
				get = function(info)
					local auras = PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER].auras
					if auras and auras[aura_name] then
						return auras[aura_name].selfonly or false
					else return false end
				end,
				set = function(info, value)
					local auras = PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER].auras
					if auras and auras[aura_name] then
						auras[aura_name].selfonly = value
						PitBull4_VisualHelpers:UpdateIndicators()
						UpdateFrames()
					end
				end,
			},

			spacercreate = {
				type = 'header',
				name = '',
				desc = '',
				order = 5,
			},
			createauraname = {
				type = 'input',
				name = L['Create new aura'],
				desc = L['Name of new aura.'],
				order = 6,
				set = function(info, value)
					local auras = PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER].auras
					local spellName = GetSpellInfo(value)
					if spellName then value = spellName end
					if not auras[value] then
						auras[value] = {}
						if(spellName) then
							local spellLink = GetSpellLink(spellName)
							print("Aura added to \""..CURRENT_HELPER.."\" helper: "..spellLink)
						else
							print("Aura added to \""..CURRENT_HELPER.."\" helper: "..value)
						end
						build_aura_list()
						PitBull4_VisualHelpers:UpdateIndicators()
						UpdateFrames()
					end
				end,
			},
			
			spacerdelete = {
				type = 'header',
				name = '',
				desc = '',
				order = 7,
			},
			delete = {
				type = 'execute',
				name = L["Delete Selected"],
				desc = L["Delete selected aura."],
				func = function(info)
					local auras = PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER].auras
					if auras[aura_name] then
						auras[aura_name] = nil
						build_aura_list()
						PitBull4_VisualHelpers:UpdateIndicators()
						UpdateFrames()
					end
				end,
				disabled = function(info)
					return (aura_name == ' ') or InCombatLockdown()
				end,
				order = 8,
			},
		},
	}
end

--PitBull4_Aura.OnProfileChanged_funcs[#PitBull4_Aura.OnProfileChanged_funcs+1] = 
--function(self)
-- Recalculate the filter options on a profile change
--	self.SetHighlightOptions(self, HIGHLIGHT_FILTER_OPTIONS)
--end
-- Generates the options for the helpers editor.
function PitBull4_VisualHelpers:GetHelpersEditor()
	local helpers = PitBull4_VisualHelpers.db.profile.global.helpers
	if not CURRENT_HELPER then
		CURRENT_HELPER = next(helpers)
	end

	build_aura_list()

	return {
		warning = {
			type = 'description',
			name = L['A reloadui is required when adding new helpers or removing existing helpers.'],
			order = 1,
		},
		helper = {
			type = 'select',
			dialogControl = MULTISELECT_CONTROL, 
			name = L['Visual Helper'],
			desc = L['Select visual helper to configure.'],
			get = function(info,key)
				if not rawget(helpers, CURRENT_HELPER) then
					CURRENT_HELPER = next(helpers)
					build_aura_list()
				end
				return CURRENT_HELPER
			end,
			set = function(info,value)
				CURRENT_HELPER = value
				build_aura_list()
				end,
			values = function(info)
				return PitBull4_VisualHelpers:GetHelpersNames()
			end,
			order = 2,
		},
		new_helper = {
			type = 'input',
			name = L['Add new visual helper'],
			desc = L['Name of new visual helper.'],
			get = function(info,key)
				return ''
			end,
			set = function(info,key,value)
				add_helper(key)
				CURRENT_HELPER = key
				build_aura_list()
				PitBull4_VisualHelpers:UpdateIndicators()
				UpdateFrames()
			end,
			validate = function(info, value)
				if value:len() < 3 then
					return L["Must be at least 3 characters long."]
				end
				if rawget(helpers, value) then
					return L["Must be unique."]
				end
				return true
			end,
			order = 3,
		},
		delete = {
			type = 'execute',
			name = L["Delete"],
			desc = L["Delete this visual helper."],
			confirm = function(info)
				LibStub("AceConfigRegistry-3.0"):NotifyChange("PitBull4")
				return L["Are you sure you want to delete the '%s' visual helper? This is irreversible."]:format(CURRENT_HELPER)
				end,
			func = function(info)
				PitBull4_VisualHelpers.db.profile.global.helpers[CURRENT_HELPER] = nil
				local h = "VisualHelpers: ".. CURRENT_HELPER

				for id, module in PitBull4:IterateModules() do
					if module.db and module.db.profile.global.helpers then
						module.db.profile.global.helpers[CURRENT_HELPER] = nil
					end
					if module.full_name == h then
						PitBull4:DisableModule(module)
						module.db.profile.global = nil
						PitBull4.db.sv.namespaces[h] = nil
					end
				end

				for name, layout in pairs(PitBull4.db.profile.layouts) do
					if layout.helpers then
						layout.helpers[CURRENT_HELPER] = nil
					end
				end

				if PitBull4.modules[h] then
					PitBull4.modules[h] = nil
					PitBull4_VisualHelpers.VH_INDICATORS[CURRENT_HELPER] = nil
				end
				PitBull4_VisualHelpers:UpdateIndicators()
				UpdateFrames()
				CURRENT_HELPER = next(helpers)
				build_aura_list()

			end,
			disabled = function(info)
				return helpers_count() <= 1 or InCombatLockdown()
			end,
			order = 4,
		},
		auras = {
			type = 'group',
			name = L['Auras'],
			desc = L['Auras selected visual helper will show.'],
			args = AURAS_GROUP,
			order = 5,
		},
	}
end
